@@ -1,7 +1,7 @@
 `adonis` <-
     function(formula, data=NULL, permutations=999, method="bray", strata=NULL,
              contr.unordered="contr.sum", contr.ordered="contr.poly",
-             ...)
+             parallel = getOption("mc.cores"), ...)
 {
     ## formula is model formula such as Y ~ A + B*C where Y is a data
     ## frame or a matrix, and A, B, and C may be factors or continuous
@@ -41,17 +41,18 @@
         dmat <- dist.lhs^2
     }
     n <- nrow(dmat)
-    I <- diag(n)
-    ones <- matrix(1,nrow=n)
-    A <- -(dmat)/2
-    G <- -.5 * dmat %*% (I - ones%*%t(ones)/n)
+    ## G is -dmat/2 centred by rows
+    G <- -sweep(dmat, 1, rowMeans(dmat))/2
     SS.Exp.comb <- sapply(H.s, function(hat) sum( G * t(hat)) )
     SS.Exp.each <- c(SS.Exp.comb - c(0,SS.Exp.comb[-nterms]) )
     H.snterm <- H.s[[nterms]]
+    ## t(I - H.snterm) is needed several times and we calculate it
+    ## here
+    tIH.snterm <- t(diag(n)-H.snterm)
     if (length(H.s) > 1)
         for (i in length(H.s):2)
             H.s[[i]] <- H.s[[i]] - H.s[[i-1]]
-    SS.Res <- sum( G * t(I-H.snterm))
+    SS.Res <- sum( G * tIH.snterm)
     df.Exp <- sapply(u.grps[-1], function(i) sum(grps==i) )
     df.Res <- n - qrhs$rank
     ## Get coefficients both for the species (if possible) and sites
@@ -66,8 +67,6 @@
     colnames(beta.sites) <- rownames(lhs)
     F.Mod <- (SS.Exp.each/df.Exp) / (SS.Res/df.Res)
 
-
-
     f.test <- function(tH, G, df.Exp, df.Res, tIH.snterm) {
       ## HERE I TRY CHANGING t(H)  TO tH, and
       ## t(I - H.snterm) to tIH.snterm, so that we don't have
@@ -91,23 +90,52 @@
           ) }
 
     ## Permutations
-    if (missing(strata))
-        strata <- NULL
-    p <- sapply(1:permutations,
-                function(x) permuted.index(n, strata=strata))
-
-
-    tH.s <- lapply(H.s, t)
-    tIH.snterm <- t(I-H.snterm)
-    ## Apply permutations for each term
-    ## This is the new f.test (2011-06-15) that uses fewer arguments
-    f.perms <- sapply(1:nterms, function(i) {
-        sapply(1:permutations, function(j) {
-            f.test(tH.s[[i]], G[p[,j], p[,j]], df.Exp[i], df.Res, tIH.snterm)
-        } )
-    })
-    ## Round to avoid arbitrary P-values with tied data
-    f.perms <- round(f.perms, 12)
+    p <- getPermuteMatrix(permutations, n, strata = strata)
+    permutations <- nrow(p)
+    if (permutations) {
+        tH.s <- lapply(H.s, t)
+        ## Apply permutations for each term
+        ## This is the new f.test (2011-06-15) that uses fewer arguments
+        ## Set first parallel processing for all terms
+        if (is.null(parallel))
+            parallel <- 1
+        hasClus <- inherits(parallel, "cluster")
+        isParal <- (hasClus || parallel > 1) && require(parallel)
+        isMulticore <- .Platform$OS.type == "unix" && !hasClus
+        if (isParal && !isMulticore && !hasClus) {
+            parallel <- makeCluster(parallel)
+        }
+        if (isParal) {
+            if (isMulticore) {
+                f.perms <-
+                    sapply(1:nterms, function(i)
+                           unlist(mclapply(1:permutations, function(j)
+                                           f.test(tH.s[[i]], G[p[j,], p[j,]],
+                                                  df.Exp[i], df.Res, tIH.snterm),
+                                           mc.cores = parallel)))
+            } else {
+                f.perms <-
+                    sapply(1:nterms, function(i)
+                           parSapply(parallel, 1:permutations, function(j)
+                                     f.test(tH.s[[i]], G[p[j,], p[j,]],
+                                            df.Exp[i], df.Res, tIH.snterm)))
+            }
+        } else {
+            f.perms <-
+                sapply(1:nterms, function(i) 
+                       sapply(1:permutations, function(j) 
+                              f.test(tH.s[[i]], G[p[j,], p[j,]],
+                                     df.Exp[i], df.Res, tIH.snterm)))
+        }
+        ## Close socket cluster if created here
+        if (isParal && !isMulticore && !hasClus)
+            stopCluster(parallel)
+        ## Round to avoid arbitrary P-values with tied data
+        f.perms <- round(f.perms, 12)
+        P <- (rowSums(t(f.perms) >= F.Mod)+1)/(permutations+1)
+    } else { # no permutations
+        f.perms <- P <- rep(NA, nterms)
+    }
     F.Mod <- round(F.Mod, 12)
     SumsOfSqs = c(SS.Exp.each, SS.Res, sum(SS.Exp.each) + SS.Res)
     tab <- data.frame(Df = c(df.Exp, df.Res, n-1),
@@ -115,12 +143,12 @@
                       MeanSqs = c(SS.Exp.each/df.Exp, SS.Res/df.Res, NA),
                       F.Model = c(F.Mod, NA,NA),
                       R2 = SumsOfSqs/SumsOfSqs[length(SumsOfSqs)],
-                      P = c((rowSums(t(f.perms) >= F.Mod)+1)/(permutations+1),
-                      NA, NA))
+                      P = c(P, NA, NA))
     rownames(tab) <- c(attr(attr(rhs.frame, "terms"), "term.labels")[u.grps],
                        "Residuals", "Total")
     colnames(tab)[ncol(tab)] <- "Pr(>F)"
-    attr(tab, "heading") <- "Terms added sequentially (first to last)\n"
+    attr(tab, "heading") <- c(howHead(attr(p, "control")),
+        "Terms added sequentially (first to last)\n")
     class(tab) <- c("anova", class(tab))
     out <- list(aov.tab = tab, call = match.call(),
                 coefficients = beta.spp, coef.sites = beta.sites,
