@@ -3,5 +3,5 @@
 {
     res <- rev(cumsum(tot.var/n:1)/n)
     names(res) <- paste("Stick", seq(len=n), sep="")
-    return(res)
+    res
 }
