@@ -1,4 +1,4 @@
-"estimateR.default" <-
+`estimateR.default` <-
     function (x, ...) 
 {
     gradF <- function(a, i) {
@@ -24,12 +24,13 @@
         Grad[1] <- .expr25/.expr26 * .expr20 + .expr8 * (1 + 
                                                          (.expr18 + a[1] * (.expr12/.expr16 - .expr13 * ((.expr7 - 
                                                                                                           .expr25 * .expr4) * .expr15 + .expr14)/.expr35)))
-        return(Grad)
+        Grad
     }
     if (!identical(all.equal(x, round(x)), TRUE)) 
         stop("function accepts only integers (counts)")
-    freq <- x[x > 0]
     X <- x[x > 0]
+    N <- sum(X)
+    SSC <- 1 # (N-1)/N # do NOT use small-sample correction
     T.X <- table(X)
     S.obs <- length(X)
     S.rare <- sum(T.X[as.numeric(names(T.X)) <= 10])
@@ -41,9 +42,28 @@
     }
     a <- sapply(i, COUNT, X)
     G <- a[1]/a[2]
-    S.Chao1 <- S.obs + a[1] * (a[1] - 1) / (a[2] + 1)/ 2
+    ## EstimateS uses basic Chao only if a[2] > 0, and switches to
+    ## bias-corrected version only if a[2] == 0. However, we always
+    ## use bias-corrected form. The switchin code is commented out so
+    ## that it is easy to put back.
+
+    ##if (a[2] > 0)
+    ##    S.Chao1 <- S.obs + SSC * a[1]^2/2/a[2]
+    ##else if (a[1] > 0)
+    ##
+    S.Chao1 <- S.obs + SSC * a[1]*(a[1]-1) / (a[2]+1)/2
+    ##else
+    ##    S.Chao1 <- S.obs
     Deriv.Ch1 <- gradF(a, i)
-    sd.Chao1 <- sqrt(a[2] * ((G^4)/4 + G^3 + (G^2)/2))
+    ##if (a[2] > 0)
+    ##    sd.Chao1 <- sqrt(a[2] * (SSC * (SSC * (G^4/4 + G^3) + G^2/2)))
+    ##else if (a[1] > 0)
+    sd.Chao1 <-
+        sqrt(SSC*(a[1]*(a[1]-1)/2/(a[2]+1) +
+                  SSC*(a[1]*(2*a[1]-1)^2/4/(a[2]+1)^2 +
+                       a[1]^2*a[2]*(a[1]-1)^2/4/(a[2]+1)^4)))
+    ##else
+    ##    sd.Chao1 <- 0
     C.ace <- 1 - a[1]/N.rare
     i <- 1:length(a)
     thing <- i * (i - 1) * a
