@@ -27,7 +27,7 @@
     ## Remove "Proportion" if only one component
     if (is.null(x$CCA) && is.null(x$pCCA))
         tbl <- tbl[,-2]
-    printCoefmat(tbl, digits = digits, na.print = "")
+    printCoefmat(tbl, digits = digits, na.print = "", zap.ind = 1:2)
     cat("Inertia is", x$inertia, "\n")
     if (!is.null(x$CCA$alias))
         cat("Some constraints were aliased because they were collinear (redundant)\n")
@@ -42,18 +42,18 @@
             "deleted due to missingness\n")
     if (!is.null(x$CCA) && x$CCA$rank > 0) {
         cat("\nEigenvalues for constrained axes:\n")
-        print(x$CCA$eig, digits = digits, ...)
+        print(zapsmall(x$CCA$eig, digits = digits), ...)
     }
     if (!is.null(x$CA) && x$CA$rank > 0) {
         ax.lim <- 8
         ax.trig <- 16
         cat("\nEigenvalues for unconstrained axes:\n")
         if (x$CA$rank > ax.trig) {
-            print(x$CA$eig[1:ax.lim], digits = digits, ...)
+            print(zapsmall(x$CA$eig[1:ax.lim], digits = digits), ...)
             cat("(Showed only", ax.lim, "of all", x$CA$rank, 
                 "unconstrained eigenvalues)\n")
         }
-        else print(x$CA$eig, digits = digits, ...)
+        else print(zapsmall(x$CA$eig, digits = digits), ...)
     }
     cat("\n")
     invisible(x)
