@@ -1,14 +1,14 @@
 `rda.default` <-
     function (X, Y, Z, scale = FALSE, ...) 
 {
-    ZERO <- 1e-04
+    ZERO <- 1e-05
     CCA <- NULL
     pCCA <- NULL
     CA <- NULL
     X <- as.matrix(X)
     NR <- nrow(X) - 1
     Xbar <- scale(X, center = TRUE, scale = scale)
-    SD <- sd(Xbar)
+    SD <- apply(Xbar, 2, sd)
     if (scale) 
         Xbar[is.nan(Xbar)] <- 0
     tot.chi <- sum(svd(Xbar, nu = 0, nv = 0)$d^2)/NR
@@ -39,7 +39,7 @@
         Y <- qr.fitted(Q, Xbar)
         sol <- svd(Y)
         ## it can happen that rank < qrank
-        rank <- min(rank, sum(sol$d > ZERO))
+        rank <- min(rank, sum(sol$d > (sol$d[1L] * ZERO)))
         sol$d <- sol$d/sqrt(NR)
         ax.names <- paste("RDA", 1:length(sol$d), sep = "")
         colnames(sol$u) <- ax.names
@@ -51,13 +51,9 @@
             CCA <- list(eig = sol$d[1:rank]^2)
             CCA$u <- as.matrix(sol$u)[, 1:rank, drop = FALSE]
             CCA$v <- as.matrix(sol$v)[, 1:rank, drop = FALSE]
-            CCA$u.eig <- sweep(as.matrix(CCA$u), 2, sol$d[1:rank], 
-                               "*")
-            CCA$v.eig <- sweep(as.matrix(CCA$v), 2, sol$d[1:rank], 
-                               "*")
-            CCA$wa.eig <- Xbar %*% sol$v[, 1:rank, drop = FALSE]
-            CCA$wa.eig <- CCA$wa.eig/sqrt(NR)
-            CCA$wa <- sweep(CCA$wa.eig, 2, 1/sol$d[1:rank], "*")
+            wa.eig <- Xbar %*% sol$v[, 1:rank, drop = FALSE]
+            wa.eig <- wa.eig/sqrt(NR)
+            CCA$wa <- sweep(wa.eig, 2, 1/sol$d[1:rank], "*")
             oo <- Q$pivot
             if (!is.null(pCCA$rank)) 
                 oo <- oo[-(1:pCCA$rank)] - ncol(Z.r)
@@ -78,8 +74,8 @@
                         QR = Q, Xbar = Xbar)
             u <- matrix(0, nrow=nrow(sol$u), ncol=0)
             v <- matrix(0, nrow=nrow(sol$v), ncol=0)
-            CCA$u <- CCA$u.eig <- CCA$wa <- CCA$wa.eig <- u
-            CCA$v <- CCA$v.eig <- v
+            CCA$u <- CCA$wa <- u
+            CCA$v <- v
             CCA$biplot <- matrix(0, 0, 0)
             CCA$alias <- colnames(Y.r)
         }
@@ -93,23 +89,19 @@
     names(sol$d) <- ax.names
     rownames(sol$u) <- rownames(X)
     rownames(sol$v) <- colnames(X)
-    rank <- min(Q$rank, sum(sol$d > ZERO))
+    rank <- min(Q$rank, sum(sol$d > (sol$d[1L] * ZERO)))
     if (rank) {
         CA <- list(eig = (sol$d[1:rank]^2))
         CA$u <- as.matrix(sol$u)[, 1:rank, drop = FALSE]
         CA$v <- as.matrix(sol$v)[, 1:rank, drop = FALSE]
-        CA$u.eig <- sweep(as.matrix(CA$u), 2, sol$d[1:rank], 
-                          "*")
-        CA$v.eig <- sweep(as.matrix(CA$v), 2, sol$d[1:rank], 
-                          "*")
         CA$rank <- rank
         CA$tot.chi <- sum(CA$eig)
         CA$Xbar <- Xbar
     } else {   # zero rank: no residual component
         CA <- list(eig = 0, rank = rank, tot.chi = 0,
                    Xbar = Xbar)
-        CA$u <- CA$u.eig <- matrix(0, nrow(sol$u), 0)
-        CA$v <- CA$v.eig <- matrix(0, nrow(sol$v), 0)
+        CA$u <- matrix(0, nrow(sol$u), 0)
+        CA$v <- matrix(0, nrow(sol$v), 0)
     }
     call <- match.call()
     call[[1]] <- as.name("rda")
