@@ -1,9 +1,17 @@
-"specpool" <-
-    function (x, pool) 
+`specpool` <-
+    function (x, pool, smallsample = TRUE) 
 {
     x <- as.matrix(x)
     if (missing(pool)) 
         pool <- rep("All", nrow(x))
+    ## check dims
+    if (length(pool) != NROW(x))
+        stop("length of 'pool' and number rows in 'x' do not match")
+    ## remove missing values
+    if (any(nas <- is.na(pool))) {
+        pool <- pool[!nas]
+        x <- x[!nas, , drop = FALSE]
+    }
     out <- seq(1:nrow(x))
     groups <- table(pool)
     inds <- names(groups)
@@ -15,6 +23,12 @@
         a1 <- a2 <- NA
         gr <- out[pool == is]
         n <- length(gr)
+        if (n <= 0)
+            next
+        if (smallsample)
+            ssc <- (n-1)/n
+        else
+            ssc <- 1
         X <- x[gr, , drop = FALSE]
         freq <- colSums(X > 0)
         p <- freq[freq > 0]/n
@@ -27,8 +41,9 @@
             a2 <- sum(freq == 2)
         else 0
         chao[is] <- S[is] + if(!is.na(a2) && a2 > 0)
-            a1 * a1/2/a2
-        else 0
+            ssc * a1 * a1/2/a2
+        else
+            ssc * a1 * (a1-1)/2
         jack.1[is] <- S[is] + a1 * (n - 1)/n
         jack.2[is] <- S[is] + a1 * (2 * n - 3)/n - a2 * (n - 
                                                          2)^2/n/(n - 1)
@@ -36,7 +51,11 @@
         aa <- if (!is.na(a2) && a2 > 0) 
             a1/a2
         else 0
-        var.chao[is] <- a2 * (0.5 + (1 + aa/4) * aa) * aa * aa
+        if (a2 > 0)
+            var.chao[is] <- a2 * ssc * (0.5 + ssc * (1 + aa/4) * aa) * aa * aa
+        else
+            var.chao[is] <-
+                ssc * (ssc * (a1*(2*a1-1)^2/4 - a1^4/chao[is]/4) + a1*(a1-1)/2)
         if (!is.na(a1) && a1 > 0) {
             jf <- table(rowSums(X[, freq == 1, drop = FALSE] > 
                                 0))
